@@ -162,7 +162,7 @@ template<class Graph,
 void r_c_shortest_paths_dispatch
 ( const Graph& g, 
   const VertexIndexMap& vertex_index_map, 
-  const EdgeIndexMap& edge_index_map, 
+  const EdgeIndexMap& /*edge_index_map*/, 
   typename graph_traits<Graph>::vertex_descriptor s, 
   typename graph_traits<Graph>::vertex_descriptor t, 
   // each inner vector corresponds to a pareto-optimal path
@@ -179,13 +179,16 @@ void r_c_shortest_paths_dispatch
   Resource_Extension_Function& ref, 
   Dominance_Function& dominance, 
   // to specify the memory management strategy for the labels
-  Label_Allocator la, 
+  Label_Allocator /*la*/, 
   Visitor vis )
 {
   pareto_optimal_resource_containers.clear();
   pareto_optimal_solutions.clear();
 
-  unsigned long i_label_num = 0;
+  typedef typename boost::graph_traits<Graph>::vertices_size_type
+    vertices_size_type;
+
+  size_t i_label_num = 0;
   typedef 
     typename 
       Label_Allocator::template rebind
@@ -217,12 +220,12 @@ void r_c_shortest_paths_dispatch
   vec_vertex_labels[vertex_index_map[s]].push_back( splabel_first_label );
   std::vector<typename std::list<Splabel>::iterator> 
     vec_last_valid_positions_for_dominance( num_vertices( g ) );
-  for( int i = 0; i < static_cast<int>( num_vertices( g ) ); ++i )
+  for( vertices_size_type i = 0; i < num_vertices( g ); ++i )
     vec_last_valid_positions_for_dominance[i] = vec_vertex_labels[i].begin();
-  std::vector<int> vec_last_valid_index_for_dominance( num_vertices( g ), 0 );
+  std::vector<size_t> vec_last_valid_index_for_dominance( num_vertices( g ), 0 );
   std::vector<bool> 
     b_vec_vertex_already_checked_for_dominance( num_vertices( g ), false );
-  while( unprocessed_labels.size() )
+  while( !unprocessed_labels.empty()  && vis.on_enter_loop(unprocessed_labels, g) )
   {
     Splabel cur_label = unprocessed_labels.top();
     unprocessed_labels.pop();
@@ -241,12 +244,12 @@ void r_c_shortest_paths_dispatch
     // if the label to be extended is undominated
     if( !cur_label->b_is_dominated )
     {
-      int i_cur_resident_vertex_num = cur_label->resident_vertex;
+      vertices_size_type i_cur_resident_vertex_num = get(vertex_index_map, cur_label->resident_vertex);
       std::list<Splabel>& list_labels_cur_vertex = 
         vec_vertex_labels[i_cur_resident_vertex_num];
-      if( static_cast<int>( list_labels_cur_vertex.size() ) >= 2 
+      if( list_labels_cur_vertex.size() >= 2 
           && vec_last_valid_index_for_dominance[i_cur_resident_vertex_num] 
-               < static_cast<int>( list_labels_cur_vertex.size() ) )
+               < list_labels_cur_vertex.size() )
       {
         typename std::list<Splabel>::iterator outer_iter = 
           list_labels_cur_vertex.begin();
@@ -318,7 +321,7 @@ void r_c_shortest_paths_dispatch
           if( !b_outer_iter_erased )
             ++outer_iter;
         }
-        if( static_cast<int>( list_labels_cur_vertex.size() ) > 1 )
+        if( list_labels_cur_vertex.size() > 1 )
           vec_last_valid_positions_for_dominance[i_cur_resident_vertex_num] = 
             (--(list_labels_cur_vertex.end()));
         else
@@ -327,7 +330,7 @@ void r_c_shortest_paths_dispatch
         b_vec_vertex_already_checked_for_dominance
           [i_cur_resident_vertex_num] = true;
         vec_last_valid_index_for_dominance[i_cur_resident_vertex_num] = 
-          static_cast<int>( list_labels_cur_vertex.size() ) - 1;
+          list_labels_cur_vertex.size() - 1;
       }
     }
     if( !b_all_pareto_optimal_solutions && cur_label->resident_vertex == t )
@@ -359,7 +362,7 @@ void r_c_shortest_paths_dispatch
       typename graph_traits<Graph>::vertex_descriptor cur_vertex = 
         cur_label->resident_vertex;
       typename graph_traits<Graph>::out_edge_iterator oei, oei_end;
-      for( tie( oei, oei_end ) = out_edges( cur_vertex, g ); 
+      for( boost::tie( oei, oei_end ) = out_edges( cur_vertex, g ); 
            oei != oei_end; 
            ++oei )
       {
@@ -409,7 +412,7 @@ void r_c_shortest_paths_dispatch
   typename std::list<Splabel>::const_iterator csi = dsplabels.begin();
   typename std::list<Splabel>::const_iterator csi_end = dsplabels.end();
   // if d could be reached from o
-  if( dsplabels.size() )
+  if( !dsplabels.empty() )
   {
     for( ; csi != csi_end; ++csi )
     {
@@ -430,8 +433,8 @@ void r_c_shortest_paths_dispatch
     }
   }
 
-  int i_size = static_cast<int>( vec_vertex_labels.size() );
-  for( int i = 0; i < i_size; ++i )
+  size_t i_size = vec_vertex_labels.size();
+  for( size_t i = 0; i < i_size; ++i )
   {
     const std::list<Splabel>& list_labels_cur_vertex = vec_vertex_labels[i];
     csi_end = list_labels_cur_vertex.end();
@@ -449,15 +452,17 @@ void r_c_shortest_paths_dispatch
 struct default_r_c_shortest_paths_visitor
 {
   template<class Label, class Graph>
-  void on_label_popped( const Label& l, const Graph& g ) {}
+  void on_label_popped( const Label&, const Graph& ) {}
   template<class Label, class Graph>
-  void on_label_feasible( const Label& l, const Graph& g ) {}
+  void on_label_feasible( const Label&, const Graph& ) {}
   template<class Label, class Graph>
-  void on_label_not_feasible( const Label& l, const Graph& g ) {}
+  void on_label_not_feasible( const Label&, const Graph& ) {}
   template<class Label, class Graph>
-  void on_label_dominated( const Label& l, const Graph& g ) {}
+  void on_label_dominated( const Label&, const Graph& ) {}
   template<class Label, class Graph>
-  void on_label_not_dominated( const Label& l, const Graph& g ) {}
+  void on_label_not_dominated( const Label&, const Graph& ) {}
+  template<class Queue, class Graph>             
+  bool on_enter_loop(const Queue& queue, const Graph& graph) {return true;}
 }; // default_r_c_shortest_paths_visitor
 
 
@@ -559,8 +564,10 @@ void r_c_shortest_paths
                                dominance, 
                                la, 
                                vis );
-  pareto_optimal_solution = pareto_optimal_solutions[0];
-  pareto_optimal_resource_container = pareto_optimal_resource_containers[0];
+  if (!pareto_optimal_solutions.empty()) {
+    pareto_optimal_solution = pareto_optimal_solutions[0];
+    pareto_optimal_resource_container = pareto_optimal_resource_containers[0];
+  }
 }
 
 // third overload:
@@ -644,8 +651,10 @@ void r_c_shortest_paths
                                dominance, 
                                default_r_c_shortest_paths_allocator(), 
                                default_r_c_shortest_paths_visitor() );
-  pareto_optimal_solution = pareto_optimal_solutions[0];
-  pareto_optimal_resource_container = pareto_optimal_resource_containers[0];
+  if (!pareto_optimal_solutions.empty()) {
+    pareto_optimal_solution = pareto_optimal_solutions[0];
+    pareto_optimal_resource_container = pareto_optimal_resource_containers[0];
+  }
 }
 // r_c_shortest_paths
 
@@ -673,7 +682,7 @@ void check_r_c_path( const Graph& g,
                      typename graph_traits<Graph>::edge_descriptor& 
                        ed_last_extended_arc )
 {
-  int i_size_ed_vec_path = static_cast<int>( ed_vec_path.size() );
+  size_t i_size_ed_vec_path = ed_vec_path.size();
   std::vector<typename graph_traits<Graph>::edge_descriptor> buf_path;
   if( i_size_ed_vec_path == 0 )
     b_feasible = true;
@@ -683,9 +692,9 @@ void check_r_c_path( const Graph& g,
         || target( ed_vec_path[0], g ) == source( ed_vec_path[1], g ) )
       buf_path = ed_vec_path;
     else
-      for( int i = i_size_ed_vec_path - 1; i >= 0; --i )
-        buf_path.push_back( ed_vec_path[i] );
-    for( int i = 0; i < i_size_ed_vec_path - 1; ++i )
+      for( size_t i = i_size_ed_vec_path ; i > 0; --i )
+        buf_path.push_back( ed_vec_path[i - 1] );
+    for( size_t i = 0; i < i_size_ed_vec_path - 1; ++i )
     {
       if( target( buf_path[i], g ) != source( buf_path[i + 1], g ) )
       {
@@ -701,7 +710,7 @@ void check_r_c_path( const Graph& g,
   b_correctly_extended = false;
   Resource_Container current_resource_levels = initial_resource_levels;
   actual_final_resource_levels = current_resource_levels;
-  for( int i = 0; i < i_size_ed_vec_path; ++i )
+  for( size_t i = 0; i < i_size_ed_vec_path; ++i )
   {
     ed_last_extended_arc = buf_path[i];
     b_feasible = ref( g, 
