@@ -11,14 +11,17 @@
 #ifndef BOOST_PROPERTY_MAP_HPP
 #define BOOST_PROPERTY_MAP_HPP
 
-#include <cassert>
+#include <boost/assert.hpp>
 #include <boost/config.hpp>
+#include <boost/static_assert.hpp>
 #include <boost/pending/cstddef.hpp>
 #include <boost/detail/iterator.hpp>
 #include <boost/concept_check.hpp>
 #include <boost/concept_archetype.hpp>
 #include <boost/mpl/assert.hpp>
 #include <boost/mpl/or.hpp>
+#include <boost/mpl/and.hpp>
+#include <boost/mpl/has_xxx.hpp>
 #include <boost/type_traits/is_same.hpp>
 
 namespace boost {
@@ -26,6 +29,39 @@ namespace boost {
   //=========================================================================
   // property_traits class
 
+  BOOST_MPL_HAS_XXX_TRAIT_DEF(key_type)
+  BOOST_MPL_HAS_XXX_TRAIT_DEF(value_type)
+  BOOST_MPL_HAS_XXX_TRAIT_DEF(reference)
+  BOOST_MPL_HAS_XXX_TRAIT_DEF(category)
+ 
+  template<class PA>
+  struct is_property_map :
+    boost::mpl::and_<
+      has_key_type<PA>,
+      has_value_type<PA>,
+      has_reference<PA>,
+      has_category<PA>
+    >
+  {};
+ 
+  template <typename PA>
+  struct default_property_traits {
+    typedef typename PA::key_type key_type;
+    typedef typename PA::value_type value_type;
+    typedef typename PA::reference reference;
+    typedef typename PA::category   category;
+  };
+ 
+  struct null_property_traits {};
+ 
+  template <typename PA>
+  struct property_traits :
+    boost::mpl::if_<is_property_map<PA>,
+      default_property_traits<PA>,
+      null_property_traits>::type
+  {};
+
+#if 0
   template <typename PA>
   struct property_traits {
     typedef typename PA::key_type key_type;
@@ -33,6 +69,7 @@ namespace boost {
     typedef typename PA::reference reference;
     typedef typename PA::category   category;
   };
+#endif
 
   //=========================================================================
   // property_traits category tags
@@ -111,6 +148,7 @@ namespace boost {
 #else
   template <class T>
   struct property_traits<T*> {
+    // BOOST_STATIC_ASSERT(boost::is_same<T, T*>::value && !"Using pointers as property maps is deprecated");
     typedef T value_type;
     typedef value_type& reference;
     typedef std::ptrdiff_t key_type;
@@ -118,6 +156,7 @@ namespace boost {
   };
   template <class T>
   struct property_traits<const T*> {
+    // BOOST_STATIC_ASSERT(boost::is_same<T, T*>::value && !"Using pointers as property maps is deprecated");
     typedef T value_type;
     typedef const value_type& reference;
     typedef std::ptrdiff_t key_type;
@@ -302,7 +341,8 @@ namespace boost {
     }
   };
 
-  struct identity_property_map;
+  template <typename T>
+  struct typed_identity_property_map;
 
   // A helper class for constructing a property map
   // from a class that implements operator[]
@@ -407,7 +447,7 @@ namespace boost {
       : iter(first), n(n_), index(_id) { }
     inline safe_iterator_property_map() { }
     inline R operator[](key_type v) const {
-      assert(get(index, v) < n);
+      BOOST_ASSERT(get(index, v) < n);
       return *(iter + get(index, v)) ;
     }
     typename property_traits<IndexMap>::value_type size() const { return n; }
@@ -539,19 +579,23 @@ namespace boost {
   };
 
   //=========================================================================
-  // A property map that applies the identity function to integers
-  struct identity_property_map
-    : public boost::put_get_helper<std::size_t, 
-        identity_property_map>
+  // A generalized identity property map
+  template <typename T>
+  struct typed_identity_property_map
+    : public boost::put_get_helper<T, typed_identity_property_map<T> >
   {
-    typedef std::size_t key_type;
-    typedef std::size_t value_type;
-    typedef std::size_t reference;
+    typedef T key_type;
+    typedef T value_type;
+    typedef T reference;
     typedef boost::readable_property_map_tag category;
 
     inline value_type operator[](const key_type& v) const { return v; }
   };
 
+//=========================================================================
+  // A property map that applies the identity function to integers
+  typedef typed_identity_property_map<std::size_t> identity_property_map;
+
   //=========================================================================
   // A property map that does not do anything, for
   // when you have to supply a property map, but don't need it.
@@ -581,6 +625,22 @@ namespace boost {
     value_type c;
   };
 
+  // Convert a Readable property map into a function object
+  template <typename PropMap>
+  class property_map_function {
+    PropMap pm;
+    typedef typename property_traits<PropMap>::key_type param_type;
+    public:
+    explicit property_map_function(const PropMap& pm): pm(pm) {}
+    typedef typename property_traits<PropMap>::value_type result_type;
+    result_type operator()(const param_type& k) const {return get(pm, k);}
+  };
+
+  template <typename PropMap>
+  property_map_function<PropMap>
+  make_property_map_function(const PropMap& pm) {
+    return property_map_function<PropMap>(pm);
+  }
 
 } // namespace boost
 
