@@ -2,7 +2,7 @@
 // v6_only.cpp
 // ~~~~~~~~~~~
 //
-// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2013 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,7 +16,9 @@
 // Test that header file is self-contained.
 #include <boost/asio/ip/v6_only.hpp>
 
-#include <boost/asio.hpp>
+#include <boost/asio/io_service.hpp>
+#include <boost/asio/ip/tcp.hpp>
+#include <boost/asio/ip/udp.hpp>
 #include "../unit_test.hpp"
 
 //------------------------------------------------------------------------------
@@ -45,9 +47,9 @@ void test()
     ip::v6_only v6_only2;
     sock.get_option(v6_only2);
     v6_only1 = true;
-    static_cast<bool>(v6_only1);
-    static_cast<bool>(!v6_only1);
-    static_cast<bool>(v6_only1.value());
+    (void)static_cast<bool>(v6_only1);
+    (void)static_cast<bool>(!v6_only1);
+    (void)static_cast<bool>(v6_only1.value());
   }
   catch (std::exception&)
   {
@@ -85,38 +87,38 @@ void test()
   {
     ip::v6_only v6_only1;
     acceptor_v6.get_option(v6_only1, ec);
-    BOOST_CHECK(!ec);
+    BOOST_ASIO_CHECK(!ec);
     bool have_dual_stack = !v6_only1.value();
 
     if (have_dual_stack)
     {
       ip::v6_only v6_only2(false);
-      BOOST_CHECK(!v6_only2.value());
-      BOOST_CHECK(!static_cast<bool>(v6_only2));
-      BOOST_CHECK(!v6_only2);
+      BOOST_ASIO_CHECK(!v6_only2.value());
+      BOOST_ASIO_CHECK(!static_cast<bool>(v6_only2));
+      BOOST_ASIO_CHECK(!v6_only2);
       acceptor_v6.set_option(v6_only2, ec);
-      BOOST_CHECK(!ec);
+      BOOST_ASIO_CHECK(!ec);
 
       ip::v6_only v6_only3;
       acceptor_v6.get_option(v6_only3, ec);
-      BOOST_CHECK(!ec);
-      BOOST_CHECK(!v6_only3.value());
-      BOOST_CHECK(!static_cast<bool>(v6_only3));
-      BOOST_CHECK(!v6_only3);
+      BOOST_ASIO_CHECK(!ec);
+      BOOST_ASIO_CHECK(!v6_only3.value());
+      BOOST_ASIO_CHECK(!static_cast<bool>(v6_only3));
+      BOOST_ASIO_CHECK(!v6_only3);
 
       ip::v6_only v6_only4(true);
-      BOOST_CHECK(v6_only4.value());
-      BOOST_CHECK(static_cast<bool>(v6_only4));
-      BOOST_CHECK(!!v6_only4);
+      BOOST_ASIO_CHECK(v6_only4.value());
+      BOOST_ASIO_CHECK(static_cast<bool>(v6_only4));
+      BOOST_ASIO_CHECK(!!v6_only4);
       acceptor_v6.set_option(v6_only4, ec);
-      BOOST_CHECK(!ec);
+      BOOST_ASIO_CHECK(!ec);
 
       ip::v6_only v6_only5;
       acceptor_v6.get_option(v6_only5, ec);
-      BOOST_CHECK(!ec);
-      BOOST_CHECK(v6_only5.value());
-      BOOST_CHECK(static_cast<bool>(v6_only5));
-      BOOST_CHECK(!!v6_only5);
+      BOOST_ASIO_CHECK(!ec);
+      BOOST_ASIO_CHECK(v6_only5.value());
+      BOOST_ASIO_CHECK(static_cast<bool>(v6_only5));
+      BOOST_ASIO_CHECK(!!v6_only5);
     }
   }
 }
@@ -125,10 +127,9 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("ip/v6_only");
-  test->add(BOOST_TEST_CASE(&ip_v6_only_compile::test));
-  test->add(BOOST_TEST_CASE(&ip_v6_only_runtime::test));
-  return test;
-}
+BOOST_ASIO_TEST_SUITE
+(
+  "ip/v6_only",
+  BOOST_ASIO_TEST_CASE(ip_v6_only_compile::test)
+  BOOST_ASIO_TEST_CASE(ip_v6_only_runtime::test)
+)
