@@ -1,6 +1,6 @@
-// --------------------------------------------------------
-//        (C) Copyright Jeremy Siek   2001.
-//        (C) Copyright Gennaro Prota 2003 - 2004.
+// -----------------------------------------------------------
+//              Copyright (c) 2001 Jeremy Siek
+//           Copyright (c) 2003-2006 Gennaro Prota
 //
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
@@ -10,9 +10,9 @@
 
 #include <fstream>
 #include <string>
-#include <cstddef> // for std::size_t
+#include <cstddef>   // for std::size_t
 #include <stdexcept> // for std::logic_error
-#include <cassert>
+#include <assert.h>
 
 #include "boost/config.hpp"
 #if !defined (BOOST_NO_STRINGSTREAM)
@@ -20,14 +20,14 @@
 #endif
 
 #include "bitset_test.hpp"
-#include "boost/dynamic_bitset.hpp"
+#include "boost/dynamic_bitset/dynamic_bitset.hpp"
 #include "boost/detail/workaround.hpp"
 
 
 // Codewarrior 8.3 for Win fails without this.
 // Thanks Howard Hinnant ;)
-#if BOOST_WORKAROUND(__MWERKS__, <= 0x3003) // 8.x
-#pragma parse_func_templ off
+#if defined __MWERKS__ && BOOST_WORKAROUND(__MWERKS__, <= 0x3003) // 8.x
+# pragma parse_func_templ off
 #endif
 
 
@@ -42,9 +42,15 @@ std::wstring widen_string( const std::string & str,
   std::wstring result;
   const std::string::size_type len = str.length();
   if(len != 0) {
+
+    typedef std::ctype<wchar_t> ct_type;
+    typedef std::wstring::traits_type tr_type;
+    const ct_type & ct = BOOST_USE_FACET(ct_type, loc);
+
     result.resize(len);
-    BOOST_USE_FACET(std::ctype<wchar_t>, loc)
-                  .widen(&str[0], 1 + &str[len-1], &result[0]);
+    for (std::size_t i = 0; i < len; ++i)
+        tr_type::assign(result[i], ct.widen(str[i]));
+
   }
   return result;
 }
@@ -128,7 +134,7 @@ void run_test_cases( BOOST_EXPLICIT_TEMPLATE_TYPE(Block) )
 
             }
             {
-              //NOTE: there are NO string stream tests - gps
+              //NOTE: there are NO string stream tests
             }
 #if !defined (BOOST_DYNAMIC_BITSET_NO_WCHAR_T_TESTS)
             {
@@ -163,7 +169,7 @@ void run_test_cases( BOOST_EXPLICIT_TEMPLATE_TYPE(Block) )
   // a good "real life" test. Some characters, such as '\v' and '\f', are not
   // used exactly because they are the ones which will most likely give problems
   // on some systems (for instance '\f' could actually be written as a sequence
-  // of new-lines, and we could never be able to read it back) [gps]
+  // of new-lines, and we could never be able to read it back)
   //
   // Note how the bitset object is not initially empty. That helps checking
   // that it isn't erroneously clear()ed by operator>>.
