@@ -1,7 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
 //
 // (C) Copyright Olaf Krzikalla 2004-2006.
-// (C) Copyright Ion Gaztanaga  2006-2007.
+// (C) Copyright Ion Gaztanaga  2006-2013.
 //
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
@@ -11,15 +11,56 @@
 //
 /////////////////////////////////////////////////////////////////////////////
 #include <boost/intrusive/detail/config_begin.hpp>
+
 #include <boost/intrusive/set.hpp>
 #include "itestvalue.hpp"
 #include "smart_ptr.hpp"
 #include "generic_set_test.hpp"
 
+namespace boost { namespace intrusive { namespace test {
+
+#if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+template<class T, class O1, class O2, class O3, class O4>
+#else
+template<class T, class ...Options>
+#endif
+struct has_insert_before<boost::intrusive::set<T,
+   #if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+   O1, O2, O3, O4
+   #else
+   Options...
+   #endif
+> >
+{
+   static const bool value = true;
+};
+
+}}}
+
+struct my_tag;
+
+using namespace boost::intrusive;
+
+template<class VoidPointer>
+struct hooks
+{
+   typedef set_base_hook
+            <void_pointer<VoidPointer> >                    base_hook_type;
+   typedef set_base_hook
+           <link_mode<auto_unlink>
+           , void_pointer<VoidPointer>
+           , tag<my_tag>
+           , optimize_size<true> >                          auto_base_hook_type;
+   typedef set_member_hook<void_pointer
+      <VoidPointer>, optimize_size<true> >                  member_hook_type;
+   typedef set_member_hook
+      <link_mode<auto_unlink>, void_pointer<VoidPointer> >  auto_member_hook_type;
+};
+
 template< class ValueType
-        , class Option1 = boost::intrusive::none
-        , class Option2 = boost::intrusive::none
-        , class Option3 = boost::intrusive::none
+        , class Option1 =void
+        , class Option2 =void
+        , class Option3 =void
         >
 struct GetContainer
 {
@@ -38,19 +79,19 @@ class test_main_template
    int operator()()
    {
       using namespace boost::intrusive;
-      typedef testvalue<VoidPointer, constant_time_size> value_type;
+      typedef testvalue<hooks<VoidPointer> , constant_time_size> value_type;
 
       test::test_generic_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::set_base_hook_t
+                  , typename hooks<VoidPointer>::base_hook_type
                   >::type
                 , GetContainer
                 >::test_all();
       test::test_generic_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::set_member_hook_t
-                               , &value_type::set_node_
+                               , typename hooks<VoidPointer>::member_hook_type
+                               , &value_type::node_
                                >
                   >::type
                 , GetContainer
@@ -66,11 +107,11 @@ class test_main_template<VoidPointer, false>
    int operator()()
    {
       using namespace boost::intrusive;
-      typedef testvalue<VoidPointer, false> value_type;
+      typedef testvalue<hooks<VoidPointer> , false> value_type;
 
       test::test_generic_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::set_base_hook_t
+                  , typename hooks<VoidPointer>::base_hook_type
                   >::type
                 , GetContainer
                 >::test_all();
@@ -78,8 +119,8 @@ class test_main_template<VoidPointer, false>
       test::test_generic_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::set_member_hook_t
-                               , &value_type::set_node_
+                               , typename hooks<VoidPointer>::member_hook_type
+                               , &value_type::node_
                                >
                   >::type
                 , GetContainer
@@ -87,7 +128,7 @@ class test_main_template<VoidPointer, false>
 
       test::test_generic_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::set_auto_base_hook_t
+                  , typename hooks<VoidPointer>::auto_base_hook_type
                   >::type
                 , GetContainer
                 >::test_all();
@@ -95,8 +136,8 @@ class test_main_template<VoidPointer, false>
       test::test_generic_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::set_auto_member_hook_t
-                               , &value_type::set_auto_node_
+                               , typename hooks<VoidPointer>::auto_member_hook_type
+                               , &value_type::auto_node_
                                >
                   >::type
                 , GetContainer
@@ -106,7 +147,8 @@ class test_main_template<VoidPointer, false>
    }
 };
 
-int main( int, char* [] ) 
+
+int main( int, char* [] )
 {
    test_main_template<void*, false>()();
    test_main_template<boost::intrusive::smart_ptr<void>, false>()();
@@ -114,4 +156,5 @@ int main( int, char* [] )
    test_main_template<boost::intrusive::smart_ptr<void>, true>()();
    return boost::report_errors();
 }
+
 #include <boost/intrusive/detail/config_end.hpp>
